from __future__ import annotations

import json
from typing import TYPE_CHECKING

from pynpm import NPMPackage

if TYPE_CHECKING:
    from pdm.backend.hooks.base import Context


def pdm_build_initialize(context: Context) -> None:
    if not context.target == "sdist":
        return

    metadata = context.config.metadata
    name = metadata["name"]
    optdeps = metadata["optional-dependencies"]
    groups = dict()
    for opt in optdeps:
        group = opt.split("-")[0]
        if group not in groups:
            groups[group] = []
        groups[group].append(opt)
    for k, v in groups.items():
        optdeps[f"all-{k}"] = [f"{name}[{','.join(v)}]"]
    optdeps["all"] = [f"{name}[{','.join('all-' + g for g in groups)}]"]

    context.config.build_config.includes.append("schilder2000/static/manifest.json")
    if "without-npm" not in context.config_settings:
        # Assume built webpack is already present otherwise
        context.ensure_build_dir()
        pkg = NPMPackage(context.root, commands=["clean-install", "run-script"])
        pkg.clean_install()
        pkg.run_script(
            "build",
            "--",
            "--output-path",
            context.build_dir / "schilder2000" / "static",
        )
        manifest_path = context.build_dir / "schilder2000" / "static" / "manifest.json"
    else:
        manifest_path = context.root / "schilder2000" / "static" / "manifest.json"
    with manifest_path.open() as fp:
        manifest: dict[str, str | dict] = json.load(fp)
    for k, v in manifest.items():
        if k == "entrypoints" or not isinstance(v, str):
            continue
        context.config.build_config.includes.append("schilder2000" + v)
